/* Copyright (C) 1999 Southern Gold Development <bruces@mail.petech.ac.za>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#define USE_DND
#include "gview.h"

void refreshSlideList(sgview *view) {
	GList *slides;
	GtkWidget *list;
	gchar *fName, *ins[1];
	gint res, cnt;
	
	if (!(view->slideList))
		return;
	list = view->slideList;
	gtk_clist_freeze(GTK_CLIST (list));
	gtk_clist_clear(GTK_CLIST (list));
	slides = view->slide;
	cnt = 0;
	while (slides) {
		fName = (gchar *) slides->data;
		if (strlen(fName) < 60)
			ins[0] = fName;
		else {
			res = strlen(fName) - 63;
			fName += res;
			ins[0] = g_strconcat("...", fName, NULL);
		}
		gtk_clist_append(GTK_CLIST (list), ins);
		gtk_clist_set_row_data(GTK_CLIST (list), cnt, (gpointer) slides);
		slides = slides->next;
		cnt++;
	}
	gtk_clist_thaw(GTK_CLIST (list));
}

void moveSlide(sgview *view, GtkCList *list, gint src, gint dest) {
	GList *tmpM, *tmpD;

	tmpM = (GList *) gtk_clist_get_row_data(list, src);
	tmpD = (GList *) gtk_clist_get_row_data(list, dest);

	/* Remove node from current position */
	if (tmpM->prev)
		tmpM->prev->next = tmpM->next;
	else
		view->slide = tmpM->next;
	if (tmpM->next)
		tmpM->next->prev = tmpM->prev;
	/* Insert node before destination */
	tmpM->prev = tmpD->prev;
	tmpM->next = tmpD;
	if (tmpD->prev)
		tmpD->prev->next = tmpM;
	else
		view->slide = tmpM;
	tmpD->prev = tmpM;
}

gint nextSlide(gpointer data) {
	sgview *view;
	GtkWidget *label;

	view = (sgview *) data;
	gtk_timeout_remove(view->timeout);
	if (!(view->slide))
		return FALSE;
	view->currSlide = view->currSlide->next;
	if (view->currSlide) {
		loadPic(view, (gchar *) view->currSlide->data);
		view->timeout = gtk_timeout_add(view->slideTime, (GtkFunction) nextSlide, (gpointer) data);
	}
	else {
		view->statCh = (view->statCh | 0x02);
		label = GTK_BUTTON (view->slideBtn)->child;
		gtk_signal_disconnect(GTK_OBJECT (view->slideBtn), view->sigSlide);
		view->sigSlide = gtk_signal_connect (GTK_OBJECT (view->slideBtn), "clicked", (GtkSignalFunc) startSlide, (gpointer) view);
		gtk_label_set_text (GTK_LABEL (label), _("SlideShow"));
	}
	return FALSE;
}

void stopSlide(GtkWidget *widget, gpointer data) {
	sgview *view;
	GtkWidget *label;

	view = (sgview *) data;
	
	if (view->currSlide) {
		view->currSlide = NULL;
		gtk_timeout_remove(view->timeout);
		view->statCh = (view->statCh | 0x02);
		label = GTK_BUTTON (view->slideBtn)->child;
		if (gtk_signal_n_emissions_by_name(GTK_OBJECT (view->slideBtn), "clicked") != 0)
			gtk_signal_emit_stop_by_name(GTK_OBJECT (view->slideBtn), "clicked");
		gtk_signal_disconnect(GTK_OBJECT (view->slideBtn), view->sigSlide);
		view->sigSlide = gtk_signal_connect (GTK_OBJECT (view->slideBtn), "clicked", (GtkSignalFunc) startSlide, (gpointer) view);
		gtk_label_set_text (GTK_LABEL (label), _("SlideShow"));
	}
}

void pauseSlide(GtkWidget *widget, gpointer data) {
	sgview *view;
	GtkWidget *label;

	view = (sgview *) data;
	
	if (view->currSlide) {
		gtk_timeout_remove(view->timeout);
		view->statCh = (view->statCh | 0x02);
		label = GTK_BUTTON (view->slideBtn)->child;
		if (gtk_signal_n_emissions_by_name(GTK_OBJECT (view->slideBtn), "clicked") != 0)
			gtk_signal_emit_stop_by_name(GTK_OBJECT (view->slideBtn), "clicked");
		gtk_signal_disconnect(GTK_OBJECT (view->slideBtn), view->sigSlide);
		view->sigSlide = gtk_signal_connect (GTK_OBJECT (view->slideBtn), "clicked", (GtkSignalFunc) startSlide, (gpointer) view);
		gtk_label_set_text (GTK_LABEL (label), _("SlideShow"));
	}
}

void resumeSlide(GtkWidget *widget, gpointer data) {
	sgview *view;
	GtkWidget *label;

	view = (sgview *) data;
	if (!(view->slide))
		return;
	if (!(view->currSlide))
		return;
	view->statCh = (view->statCh & 0xfd);
	loadPic(view, (gchar *) view->currSlide->data);
	label = GTK_BUTTON (view->slideBtn)->child;
	gtk_label_set_text(GTK_LABEL (label), _("Stop"));
	if (gtk_signal_n_emissions_by_name(GTK_OBJECT (view->slideBtn), "clicked") != 0)
		gtk_signal_emit_stop_by_name(GTK_OBJECT (view->slideBtn), "clicked");
	gtk_signal_disconnect(GTK_OBJECT (view->slideBtn), view->sigSlide);
	view->sigSlide = gtk_signal_connect(GTK_OBJECT (view->slideBtn), "clicked", (GtkSignalFunc) stopSlide, (gpointer) view);
	view->timeout = gtk_timeout_add(view->slideTime, (GtkFunction) nextSlide, (gpointer) data);
}

void clearSlide(GtkWidget *widget, gpointer data) {
	sgview *view;
	GList *tmp;
	
	view = (sgview *) data;
	if (!(view->statCh & 0x02))
		return;
	if (view->slide) {
		tmp = view->slide;
		while (tmp) {
			g_free(tmp->data);
			tmp = tmp->next;
		}
		g_list_free(tmp);
		view->slide = NULL;
		view->currSlide = NULL;
	}
	view->currSlide = NULL;
	refreshSlideList(view);
}

void startSlide(GtkWidget *widget, gpointer data) {
	sgview *view;
	GtkWidget *label;

	view = (sgview *) data;
	if (!(view->slide))
		return;
	view->currSlide = view->slide;
	view->statCh = (view->statCh & 0xfd);
	loadPic(view, (gchar *) view->currSlide->data);
	label = GTK_BUTTON (view->slideBtn)->child;
	gtk_label_set_text(GTK_LABEL (label), _("Stop"));
	if (gtk_signal_n_emissions_by_name(GTK_OBJECT (view->slideBtn), "clicked") != 0)
		gtk_signal_emit_stop_by_name(GTK_OBJECT (view->slideBtn), "clicked");
	gtk_signal_disconnect(GTK_OBJECT (view->slideBtn), view->sigSlide);
	view->sigSlide = gtk_signal_connect(GTK_OBJECT (view->slideBtn), "clicked", (GtkSignalFunc) stopSlide, (gpointer) view);
	view->timeout = gtk_timeout_add(view->slideTime, (GtkFunction) nextSlide, (gpointer) data);
}

void queueSlide(sgview *view, const gchar* fname) {
	if (view->slide) {
		g_list_append(view->slide, (gpointer) fname);
	}
	else {
		view->slide = g_list_alloc();
		view->slide->data = (gpointer) fname;
	}
}

void enqueueSlide(GtkWidget *widget, gpointer data) {
	sgview *view;
	gchar *file;
	GList *selList;
	
	view = (sgview *) data;
	if (GTK_CLIST (view->clist)->focus_row == -1)
		return;
	if (GTK_CLIST (view->clist)->selection) {
		selList = GTK_CLIST (view->clist)->selection;
		while (selList) {
			gtk_clist_get_text(GTK_CLIST (view->clist), (int) selList->data, 0, &file);
			if ((strcmp(file, _(".. (parent)")) != 0) && (file[strlen(file) - 1] != '/')) {
				queueSlide(view, g_strconcat(view->currPath, file, NULL));
			}
			selList = selList->next;
		}
	}
	else {
		gtk_clist_get_text(GTK_CLIST (view->clist), GTK_CLIST(view->clist)->focus_row, 0, &file);
		if ((strcmp(file, _(".. (parent)")) != 0) && (file[strlen(file) - 1] != '/')) {
			queueSlide(view, g_strconcat(view->currPath, file, NULL));
		}
	}
	refreshSlideList(view);
}

void viewSlideKill(GtkWidget *widget, gpointer data) {
	ren *viewD;
	
	viewD = (ren *) data;
	viewD->view->slideList = NULL;
	viewD->view = NULL;
	g_free(viewD);
}

void viewSlideDestroy(GtkWidget *widget, gpointer data) {
	ren *viewD;
	
	viewD = (ren *) data;
	gtk_widget_destroy(viewD->rsp);
}

void loadSlide(ren *viewD) {
	gint cnt;
	
	cnt = 0;
	viewD->view->currSlide = (GList *) gtk_clist_get_row_data(GTK_CLIST (viewD->ent), GTK_CLIST (viewD->ent)->focus_row);
	loadPic(viewD->view, (gchar *) viewD->view->currSlide->data);
}

gint viewSlideMouse(GtkWidget *widget, GdkEventButton *event, gpointer data) {
	ren *viewD;
	
	viewD = (ren *) data;
	if (GTK_CLIST (viewD->ent)->focus_row == -1)
		return FALSE;
	switch (event->button) {
		case 1:
			if (event->type == GDK_2BUTTON_PRESS) 
				loadSlide(viewD);
			break;
		default:
			break;
	} 
	return FALSE;
}

gint viewSlideKey(GtkWidget *widget, GdkEventKey *event, gpointer data) {
	ren *viewD;
	
	viewD = (ren *) data;
	if (GTK_CLIST (viewD->ent)->focus_row == -1)
		return FALSE;

	switch (event->keyval) {
		case GDK_Return:
		case GDK_KP_Enter:
			loadSlide(viewD);
			break;
		case GDK_q:
		case GDK_Q:
			viewSlideDestroy(widget, data);
			break;
		default:
			break;
	}
	return FALSE;
}

void viewSlide(GtkWidget *widget, gpointer data) {
	ren *viewD;
	GtkWidget *rsp, *scr, *list, *fix, *hbox;
	gchar *titles[] = {_("File")};
	
	viewD = g_malloc(sizeof(ren) * 1);
	viewD->view = (sgview *) data;
	
	rsp = gtk_window_new(GTK_WINDOW_TOPLEVEL);
	viewD->rsp = rsp;
	gtk_window_set_title(GTK_WINDOW (rsp), "Files In Slide List");
	gtk_window_set_policy(GTK_WINDOW (rsp), FALSE, FALSE, FALSE);
	gtk_window_set_position(GTK_WINDOW (rsp), GTK_WIN_POS_MOUSE);
	gtk_widget_set_usize(GTK_WIDGET (rsp), 380, 400);
	gtk_signal_connect(GTK_OBJECT (rsp), "destroy", (GtkSignalFunc) viewSlideKill, (gpointer) viewD);
	gtk_container_border_width(GTK_CONTAINER (rsp), 5);

	fix = gtk_fixed_new();
	gtk_container_add(GTK_CONTAINER (rsp), fix);
	
	scr = gtk_scrolled_window_new(NULL, NULL);
	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW (scr), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
	gtk_widget_set_usize(GTK_WIDGET (scr), 370, 360);
	gtk_fixed_put(GTK_FIXED (fix), scr, 0, 0);
	
	list = gtk_clist_new_with_titles(1, titles);
	viewD->ent = list;
	viewD->view->slideList = list;
	gtk_drag_dest_set(list, GTK_DEST_DEFAULT_ALL, target_table, 3, GDK_ACTION_COPY | GDK_ACTION_MOVE);
	gtk_drag_source_set(list, GDK_BUTTON1_MASK, target_table, 2, GDK_ACTION_COPY | GDK_ACTION_MOVE);
	gtk_signal_connect(GTK_OBJECT (list), "drag_data_received", (GtkSignalFunc) dropDragSlide, (gpointer) viewD->view);
	gtk_signal_connect(GTK_OBJECT (list), "drag_data_get", (GtkSignalFunc) startDragSlide, (gpointer) viewD);
	gtk_clist_set_column_width(GTK_CLIST (list), 0, 310);
	gtk_clist_column_titles_passive(GTK_CLIST (list));
	gtk_clist_set_column_justification(GTK_CLIST (list), 0, GTK_JUSTIFY_LEFT);
	gtk_clist_set_auto_sort(GTK_CLIST (list), FALSE);
	gtk_clist_set_selection_mode(GTK_CLIST (list), GTK_SELECTION_SINGLE);
	gtk_signal_connect_after(GTK_OBJECT (list), "button_press_event", (GtkSignalFunc) viewSlideMouse, (gpointer) viewD);
	gtk_signal_connect(GTK_OBJECT (list), "key_press_event", (GtkSignalFunc) viewSlideKey, (gpointer) viewD);
	gtk_container_add(GTK_CONTAINER (scr), list);
	gtk_widget_show(list);
	gtk_widget_show(scr);

	hbox = gtk_hbox_new(TRUE, 5);
	gtk_widget_set_usize(GTK_WIDGET (hbox), 370, 25);
	gtk_fixed_put(GTK_FIXED (fix), hbox, 0, 365);

	gAddButtonBox(viewD->view, (gpointer) viewD, GTK_BOX (hbox), _("Dismiss"), _("Dismiss Slide List"), (GtkSignalFunc) viewSlideDestroy);
	gAddButtonBox(viewD->view, (gpointer) viewD->view, GTK_BOX (hbox), _("Clear"), _("Clear Slide List"), (GtkSignalFunc) clearSlide);

	gtk_widget_show(hbox);
	gtk_widget_show(fix);
	gtk_widget_show(rsp);
	refreshSlideList(viewD->view);
}
